from .eight_plus_sixteen import *  # noqa
from .five_plus_ten import *  # noqa
from .four_plus_eight import *  # noqa
from .six_plus_twelve import *  # noqa
from .ten_plus_twenty import *  # noqa
from .three_plus_six import *  # noqa
from .two_plus_four import *  # noqa
