from .eight_plus_twelve import *  # noqa
from .four_plus_six import *  # noqa
from .four_plus_six_2 import *  # noqa
from .six_plus_nine import *  # noqa
from .twenty_plus_thirty import *  # noqa
from .two_plus_three import *  # noqa
